unit uUploadForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, HttpUploadThreadEx;

type
  TUploadForm = class(TForm)
    ProgressBar: TProgressBar;
    UploadLabel: TLabel;
    ThLabel: TLabel;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    HttpUploadThread: THttpUploadThreadEx;
    FFileName: String;
    procedure SetFileName(AFileName: String);
    procedure WorkBegin;
    procedure WorkEnd;
    procedure Work;
    procedure DoTerminate(Sender: TObject);
  public
    property FileName: String read FFileName write SetFileName;
  end;

var
  UploadForm: TUploadForm;

implementation

{$R *.dfm}

uses
  IdComponent, uLogger;

resourcestring
  c_uploading = 'Uploading: %d %%';
  c_uploading_done = 'Uploading: Done';
  c_thanks = 'Thank you for uploading!';

procedure TUploadForm.FormCreate(Sender: TObject);
begin
  HttpUploadThread := THttpUploadThreadEx.Create;
  with HttpUploadThread do
  begin
    FreeOnTerminate := True;
    OnTerminate := DoTerminate;
    OnWorkBegin := WorkBegin;
    OnWorkEnd := WorkEnd;
    OnWork := Work;
  end;
  HttpUploadThread.Logger := Logger;

  ThLabel.Caption := c_thanks;
end;

procedure TUploadForm.FormShow(Sender: TObject);
begin
  if Assigned(HttpUploadThread) then
    HttpUploadThread.Resume;
end;

procedure TUploadForm.SetFileName(AFileName: String);
begin
  FFileName := AFileName;
  HttpUploadThread.FileName := FFileName;
end;

procedure TUploadForm.WorkBegin;
begin
  if not Assigned(HttpUploadThread) then
    Exit;
  if HttpUploadThread.WorkMode = wmWrite then
  begin
    ProgressBar.Position := 0;
    ProgressBar.Max := HttpUploadThread.WorkCountMax;
  end;
end;

procedure TUploadForm.WorkEnd;
begin
  if not Assigned(HttpUploadThread) then
    Exit;

  if HttpUploadThread.WorkMode = wmWrite then
    UploadLabel.Caption := c_uploading_done;
end;

procedure TUploadForm.Work;
begin
  if not Assigned(HttpUploadThread) then
    Exit;

  if HttpUploadThread.WorkMode = wmWrite then
  begin
    ProgressBar.Position := HttpUploadThread.WorkCount;
    UploadLabel.Caption := Format(c_uploading, [Round (100 * ProgressBar.Position / ProgressBar.Max)]);
  end;
end;

procedure TUploadForm.DoTerminate(Sender: TObject);
begin
  Self.Close;
end;

procedure TUploadForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
{ FreeOnTerminate <- False
  if Assigned(HttpUploadThread) then
  begin
    HttpUploadThread.Terminate;
    HttpUploadThread.Wait;
    HttpUploadThread.Free;
  end;
}
  Action := caFree;
end;

end.
